// C Source File
// Created 27/07/2003; 12:02:21

#include "mkcomplib.h"

/*===============================================================================================
== Komp compression of a variable in an other (or the same) variable
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== name_var_dest : name of the variable where the compressed datas will be stored
         if it's NULL, compress in the same variable
== error : pointer to return the error code
==================================================================================================
== return the hsym structure created, else HS_NULL
===============================================================================================*/
HSym komp_hsym_var(HSym hsym_var_src, const unsigned char * name_var_dest, CompErr * error) {
  HANDLE h_var;
  unsigned char * data;
  unsigned short length;
  short type;

  unsigned short * stat=NULL;
  TREE * tree=NULL;
  unsigned short comp_len;
  short offset;
  short nb1;
  short nb2;
  short mode;

  HANDLE h;
  unsigned char * dest;
    
  if (hsym_var_src.folder==H_NULL) {
    *error=COMP_ERR_NO_SRC_VAR;
    return HS_NULL;
  }
  
  h_var=DerefSym(hsym_var_src)->handle;
  data=HLock(h_var);
  if (data==NULL) {
    *error=COMP_ERR_MEM;
    return HS_NULL;
  }
  length=*(unsigned short *)data;

  type=data[length+1];

  *error=study_komp(data+2,length,&stat,&tree,&comp_len,&offset,&nb1,&nb2,&mode,FALSE);
  if (*error!=COMP_NO_ERR) {
    HeapUnlock(h_var);
    return HS_NULL;
  }

  h=HeapAlloc(comp_len+2);
  if (h==H_NULL) {
    free(tree);
    free(stat);
    *error=COMP_ERR_MEM;
    return HS_NULL;
  }
  
  dest=HLock(h);

  *error=mk_komp(data+2,length,dest,stat,tree,comp_len,offset,nb1,nb2,mode,type,FALSE);
  
  if (*error!=COMP_NO_ERR) {
    HeapFree(h);
    return HS_NULL;
  }
  
  HeapUnlock(h_var);
  if (*error!=COMP_NO_ERR)
    return HS_NULL;

  return mk_var(hsym_var_src,name_var_dest,h,error);

}

